import { RetryStrategyOptions } from 'redis';
import config from '@config';
import RedisPlugin from '@/plugins/redis';
import MemoryCachePlugin from '@/plugins/memoryCache';
import LocalDbPlugin from '@/plugins/localDb';
import MyLogger, { interfaceLog } from '@/plugins/logger';

export type Redis = RedisPlugin
export type MemoryCache = MemoryCachePlugin
export type LocalDb = LocalDbPlugin
export type Log = interfaceLog
const { redis: redisOpt, localDb: localDbOpt } = config
const redisOption = {
  host: redisOpt.host,
  port: redisOpt.port,
  password: redisOpt.password,
  auth_pass: redisOpt.password,
  retry_strategy: function(this: RedisPlugin, opts: RetryStrategyOptions) {
    if (opts.error && opts.error.code === "ECONNREFUSED") {
      opts.error.code
      // redis连接失败时使用内存存储
      this.isUseRedis = false
      // End reconnecting on a specific error and flush all commands with
      // a individual error
      return new Error("The server refused the connection");
    }
    if (opts.total_retry_time > 1000 * 60 * 60) {
      // End reconnecting after a specific timeout and flush all commands
      // with a individual error
      return new Error("Retry time exhausted");
    }
    if (opts.attempt > 10) {
      // End reconnecting with built in error
      return;
    }
    // reconnect after
    return Math.min(opts.attempt * 1000, 3000);
  }
}
const localDbOption = localDbOpt
export default () => {
  const redis = new RedisPlugin(redisOption)
  const memoryCache = new MemoryCachePlugin()
  const localDb = new LocalDbPlugin(localDbOption)
  const logger = new MyLogger()
  return {
    redis,
    memoryCache,
    db: localDb,
    logger
  }
}
